# -*- coding: utf-8 -*-

from zope.component import getUtility, getMultiAdapter
from zope.component import getSiteManager
from plone.portlets.interfaces import *
from zope.app.container.interfaces import INameChooser
from plone.app.portlets.storage import UserPortletAssignmentMapping
from plone.portlets.constants import CONTEXT_CATEGORY, USER_CATEGORY
from plone.portlets.registration import PortletType
from plone.portlets.interfaces import IPortletType
from zope.component import Interface

def get_assignment(context, name):
    """
    Get portlet assignment for this level
    """
    manager = getUtility(IPortletManager, name=name)        

    assignable = getMultiAdapter((context, manager,), ILocalPortletAssignmentManager)
    assignment = getMultiAdapter((context, manager), IPortletAssignmentMapping).__of__(context)

    return assignable, assignment

def set_portlets(context, name, portlets, block_parent = True, clear = True):
    """
    Set portlets at given level
    """
    assignable, assignment = get_assignment(context, name)
    assignable.setBlacklistStatus(CONTEXT_CATEGORY, block_parent)    

    if clear:
        for key in assignment.keys():
            del assignment[key]

    for portlet in portlets:
        add_portlet(context, name, portlet)

def add_portlet(context, name, portlet):
    """
    Add a portlet, if it doesn't exist already
    """
    assignable, assignment = get_assignment(context, name)

    for key in assignment.keys():
        if assignment[key].__class__ == portlet.__class__:
            return        

    chooser = INameChooser(assignment)
    assignment[chooser.chooseName(None, portlet)] = portlet

def remove_portlet(context, name, portlet):
    """
    Del a portlet, if it exists already
    """
    assignable, assignment = get_assignment(context, name)

    for key in assignment.keys():
        if assignment[key].__class__ == portlet.__class__:
            del assignment[key]

def setup_user_dashboard(dashboard, username, values, order = None):
    """
    Setup user dashboard, for example:
        from plone.app.portlets.portlets import recent
        utils.setup_user_dashboard('plone.dashboard3',
                                   userid,
                                   { "diary": recent.Assignment() })
    """
    
    dashboard_obj = getUtility(IPortletManager, name=dashboard)
    users = dashboard_obj["user"]

    user = users.get(username, None)
    if user is not None:
        del users[username]
    user = UserPortletAssignmentMapping(manager=dashboard,
                                        category=USER_CATEGORY,
                                        name=username)    
    users[username] = user

    if not order:
        order = values.keys()
    for key in order:
        user[key] = values[key]
    

def installViewlet(self, manager, skinname, name):
    """
    Install a viewlet
    """
    storage = queryUtility(IViewletSettingsStorage)
    skins = getattr(storage, '_order')
    values = list(skins[skinname].get(manager, []))
    values.append(viewlet_name)
    storage.setOrder(manager, skinname, tuple(values))
    
    
def installPortlet(self, addview, title, description=""):
    """
    Install a portlet
    """
    sm = getSiteManager(self)

    portlet = PortletType()
    portlet.title = title
    portlet.description = description
    portlet.addview = addview
    portlet.for_ = [ Interface ]

    sm.registerUtility(component=portlet,
                       provided=IPortletType,
                       name=addview)
    
